/*-----------------------------------------------------------------------
 * Copyright 1999 Michael Lincoln <mclinc@essex.ac.uk>
 * 
 * This software is provided free of charge.  Permission is hereby given
 * to use, copy, modify and distribute free of charge the software for
 * any purpose provided that this notice is included verbatim in every
 * copy.  The software is provided on an "as is" basis and the authors
 * and the University of Essex have no obligation to provide maintenance,
 * support, updates, enhancements, or modifications.
 * 
 * All express or implied warranties whether statutory or otherwise as to
 * the merchantability or fitness for a particular purpose of the
 * software are, to the maximum extent permitted by law, excluded.
 *
 * In no circumstances whatsoever shall the author or the University of
 * Essex be liable in contract, tort, or otherwise including liability
 * for negligence for any loss whether direct, indirect or otherwise of
 * business, revenue, profits, anticipated savings, wasted expenditure,
 * corruption or destruction of data or for any indirect or consequential
 * loss whatsoever.
 * 
 *
 *------------------------------------------------------------------------
 */




#include <dmedia/vl.h>
#include <stdio.h>
#include "hvci.h"
/*
  HVCI

 The irix version for SGI INDYs

 Mike Lincoln 1998


*/


static VLServer svr;
static VLNode src, drn;
static VLControlValue val;
static VLBuffer buffer;
static VLInfoPtr info;
static VLPath path;
static unsigned char* imageBuffer;

static int currentWidth,currentHeight;
static int convertFormat;
static int maxWidth,maxHeight;
static int video_type;
static int videoDataFormat;


/* Report errors */
void error_exit(void)

{
    char *message = "hcvi: error_exit,  Error!\n";
    vlPerror(message);
    exit(1);
}



/*
  Open the win tv device
  mmap the bttv memory.
*/


void video_initalize(int vidtype)

{


  if(vidtype == VIDEO_PAL)
    {   
      maxWidth  = 768;
      maxHeight = 576;
    }

  else if (vidtype == VIDEO_NTSC)
    {
      /* these values are not correct!!!!!!! --only a guess!*/
      maxWidth  = 640; 
      maxHeight = 525;
    }

  else
    {
      fprintf(stderr,"hvci: initalize,  Invalid video type\n");
      exit(1);
    }
  
  

  video_type = vidtype;




  
    /* Connect to the daemon */
    if (!(svr = vlOpenVideo(""))) 
	error_exit();


    /* Set up a drain node in memory */
    drn = vlGetNode(svr, VL_DRN, VL_MEM, VL_ANY);
    
    /* Set up a source node on any video source  */
    src = vlGetNode(svr, VL_SRC, VL_VIDEO, VL_ANY);

    /* Create a path using the first device that will support it */
    path = vlCreatePath(svr, VL_ANY, src, drn); 

    /* Set up the hardware for and define the usage of the path */
    if ((vlSetupPaths(svr, (VLPathList)&path, 1, VL_SHARE, VL_SHARE)) < 0)
	error_exit();

    /* Set the packing to RGB */
    val.intVal = VL_PACKING_RGB_8;
    vlSetControl(svr, path, drn, VL_PACKING, &val);
    
    /* Convert the fromat in the grab function*/
    convertFormat = TRUE;
    videoDataFormat = VIDEO_24_RGB;


    /* Create and register a buffer for 1 frame */
    buffer = vlCreateBuffer(svr, path, drn, 1);
    
    if (buffer == NULL)
	error_exit();	
    
    vlRegisterBuffer(svr, path, drn, buffer);


    /* Get the video size */
    vlGetControl(svr, path, drn, VL_SIZE, &val);
    
    currentWidth = val.xyVal.x;
    currentHeight = val.xyVal.y;
    
    printf("Vl size is %i , %i\n",currentWidth,currentHeight);
    

    /* allicate an image buff which we will return pointers to.*/
    
    imageBuffer = (unsigned char*) malloc(maxWidth*maxHeight*4*sizeof(unsigned char));
    

    if(!imageBuffer)
      {
	fprintf(stderr,"hvci,initalize: Memory allication error\n");
	exit(1);
      }


}

/*
 Change the size of the image
*/
  
void video_set_scale(float denom)
{
    
    /* 
       The scale factor is a fuction of the maximum video size for your area.
       eg for 1/2 size PAL the denom would be set to 2, this would give
       768/2 x 576/2.
    */
    
    if (denom == 0)
	{
	    fprintf(stderr,"hvci: video_set_scale, infinitely  large images not supported.\n");
	    return;
	}
    
    
    currentWidth = maxWidth/(int)denom;
    currentHeight = maxHeight/(int)denom;
  
    /* 
       Set the zoom factor which scales the video under irix.
       Many settings dont work, maybe there not supported?
    */
    
    val.fractVal.numerator = 1;
    val.fractVal.denominator =  (int) denom;
    
    if (vlSetControl(svr,path, drn,VL_ZOOM, &val))
	{
	    vlPerror("hvci:video_set_scale, Unable to set zoom\n");
	    
	}
    
    /*
      Set the video area which will be used to crop our zoomed image
    */
    
    val.xyVal.x = currentWidth;
    val.xyVal.y = currentHeight;
    
    if (vlSetControl(svr, path, drn,VL_SIZE, &val))
	{
	    vlPerror( "hvci:video_set_scale, Unable to set crop size\n");
	    
	}
    
    
    /* Get the video size */
    vlGetControl(svr, path, drn, VL_SIZE, &val);
    
    currentWidth = val.xyVal.x;
    currentHeight = val.xyVal.y;
        
}

int video_get_width()
{
  /*
    just return the width currently 
  */
    
  return currentWidth;
}

int video_get_height()
{
    /*
      just return the height currently
    */
    
    return currentHeight;
}




/*
  tidys up open files and maped memory etc.
*/


void video_tidyup()
{
       
    free(imageBuffer);
    
    
    /* Cleanup before exiting */
    vlDeregisterBuffer(svr, path, drn, buffer);
    vlDestroyBuffer(svr, buffer);
    vlDestroyPath(svr, path);
    vlCloseVideo(svr);
    
}


/*
  Grabs a  image 
*/

unsigned char *video_grab()
{
  int index;

  /* The colour format for the SGI inVL_PACKING_RGB_8 
    mode is is XRGB, This is going to need some converting...
  */

 

    unsigned char* dataPtr;
    unsigned char* ImagePtr = imageBuffer;
    short int * short_ImagePtr = (short int*) imageBuffer;
    int r,g,b;

  
    /* Begin the data transfer */
    if (vlBeginTransfer(svr, path, 0, NULL))
	error_exit();

    /* Wait for a frame */
    do {
    	info = vlGetNextValid(svr, buffer);
    } while (!info);

  
     
    /* Get a pointer to the frame */
    dataPtr = vlGetActiveRegion(svr, buffer, info);
   
    /*convert it form 32 XBRG to be V4I  like 24 BRG compatible*/
    if(videoDataFormat == VIDEO_24_RGB)
	for(index=0;index<(currentWidth*currentHeight);index++)
	    {
		/*
		  This may (is) not the most efficent way of doing it.
		*/
		
		dataPtr++;
		*(ImagePtr++) = *(dataPtr++);
		*(ImagePtr++) = *(dataPtr++);
		*(ImagePtr++) = *(dataPtr++);
		
	    }
    else  if (videoDataFormat == VIDEO_8_GRAY)
	/* just copy the 1 st widthxheight pixels */
	for(index=0;index<(currentWidth*currentHeight);index++)
	    {
		
		*(ImagePtr++) = *(dataPtr++);
		
	    }
    else if (videoDataFormat == VIDEO_32_RGB)
	
	for(index=0;index<(currentWidth*currentHeight);index++)
	    
	    {
		/*
		  This could be done in 32bit chunks but i cant be bothered,as
		  this will be removed at some point
		*/
		
		*(ImagePtr++) = *(dataPtr++);
		*(ImagePtr++) = *(dataPtr++);
		*(ImagePtr++) = *(dataPtr++);
		*(ImagePtr++) = *(dataPtr++);
	    }
    
    
    else if (videoDataFormat = VIDEO_16_RGB)	
      for(index=0;index<(currentWidth*currentHeight);index++)
	
	{
	  /*
	    convert 32 bit to 16 bit 565 format
	  */
	      
	  
	  
	  dataPtr++;
	  r = (*(dataPtr++)/8);
	  g = (*(dataPtr++)/4)<<5;
	  b = (*(dataPtr++)/8)<<11;
	  
	  *(short_ImagePtr++) = (short int ) r|g|b;           
	  
	  
	}
    

    /* Finished with frame, unlock the buffer */
    vlPutFree(svr, buffer);
    
    /* End the data transfer */
    vlEndTransfer(svr, path);
    /*Return the Generic image Format*/

    return imageBuffer;
}


/*
  Double buffered grabbing
*/

unsigned char *video_db_grab()
{
  /*emulate it using single buffered grabing for now...*/


  return video_grab();    
  
}


void video_set_source(int source)
{
  fprintf(stderr,"video_set_source: Operation currently unsupported on SGIs, Using value last set in video panel.\n");

}


/*
  Set the video format for the image data
*/



void video_set_format(unsigned int format)
{

  /* THIS SHOULD BE A CASE STATEMENT!!!*/
  
  if(format == VIDEO_8_GRAY)
    {
      
      /* Set the packing to 8 bit gray scale */
      val.intVal = VL_PACKING_Y_8_P ;
      vlSetControl(svr, path, drn, VL_PACKING, &val);
      
      
      /* format is native, no converstion nessasary*/
      convertFormat = FALSE;
      videoDataFormat = VIDEO_8_GRAY;
    }
  else
  if(format == VIDEO_16_RGB)
    {
      
      /* Set the packing to RGB */
      val.intVal = VL_PACKING_RGB_8;
      vlSetControl(svr, path, drn, VL_PACKING, &val);
      
      /* Grab needs to convert it after*/
      convertFormat = TRUE;
      videoDataFormat = VIDEO_16_RGB;
      
    }
  else
  if(format == VIDEO_24_RGB)
    {
      
      /* Set the packing to RGB */
      val.intVal = VL_PACKING_RGB_8;
      vlSetControl(svr, path, drn, VL_PACKING, &val);
      
      /* Grab needs to convert it after*/
      convertFormat = TRUE;
      videoDataFormat = VIDEO_24_RGB;
      
    }
  else
  if(format == VIDEO_32_RGB)
    { 
      
      /* Set the packing to RGB */
      val.intVal = VL_PACKING_RGB_8;
      vlSetControl(svr, path, drn, VL_PACKING, &val);
      
      
      /* format is native, no converstion nessasary*/
      convertFormat = FALSE;
      videoDataFormat = VIDEO_32_RGB;
    }
  else
    {
      fprintf(stderr,"hvci:set_format, Unknow video format.\n");
      exit(1);
    }
  
}
