/*-----------------------------------------------------------------------
 * Copyright 1999 Michael Lincoln <mclinc@essex.ac.uk>
 * 
 * This software is provided free of charge.  Permission is hereby given
 * to use, copy, modify and distribute free of charge the software for
 * any purpose provided that this notice is included verbatim in every
 * copy.  The software is provided on an "as is" basis and the authors
 * and the University of Essex have no obligation to provide maintenance,
 * support, updates, enhancements, or modifications.
 * 
 * All express or implied warranties whether statutory or otherwise as to
 * the merchantability or fitness for a particular purpose of the
 * software are, to the maximum extent permitted by law, excluded.
 *
 * In no circumstances whatsoever shall the author or the University of
 * Essex be liable in contract, tort, or otherwise including liability
 * for negligence for any loss whether direct, indirect or otherwise of
 * business, revenue, profits, anticipated savings, wasted expenditure,
 * corruption or destruction of data or for any indirect or consequential
 * loss whatsoever.
 * 
 *
 *------------------------------------------------------------------------
 */




/*

  hvci.c
  
  Low level  C grabbing code 
  
  
  Suports only simple operations, will exit(1) with message to stderr on 
   errors.
  

  Mike Lincoln 1998


*/


#include <unistd.h>
#include <sys/mman.h>
#include <sys/ioctl.h> 
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>


#include "hvci.h"

/* This will be in the kernel at some point -as of now 8-) 2.2.x kernels*/

#include <linux/videodev.h>

/*
  File discriptor for video capture device and mmaped memory and settings
*/

static int fd;
static unsigned char *map;
static struct video_mmap  gb;
static struct video_mbuf mbuf;
static struct video_picture vpict;
static int video_type;
static int maxWidth,maxHeight;
static int currentWidth,currentHeight;

/*
  Open the win tv device
  mmap the bttv memory.
*/


void video_initalize(int vidtype)
{
 
    /*this is now officaly video0,but i'll be sym-linked*/
    char *device_name = "/dev/bttv";

    if(vidtype == VIDEO_PAL)
	{   
	    maxWidth  = 768;
	    maxHeight = 576;
	}
    
    else if (vidtype == VIDEO_NTSC)
	{
	    /* these values are not correct!!!!!!!*/
	    maxWidth  = 640; 
	    maxHeight = 525;
	}
    
    else
	{
 	    fprintf(stderr,"hvci: initalize,  Invalid video type\n");
	    exit(1);
	}
    
  

    video_type = vidtype;




    /* Attempt to open the device */
    
    if (-1 == (fd = open(device_name,O_RDWR)))
	{
	    fprintf(stderr,"hvci: initalize,Cannot open %s\n", device_name);
	    exit(1);
	}
    
    
    /* 
       Now setup shared memory, ask the driver how much memory I need to map.
    */
  
    
    if(-1 == ioctl(fd,VIDIOCGMBUF,&mbuf))
	{
	    fprintf(stderr,"hvci: initalize, ioctl VIDIOCGMBUF error\n");
	    exit(1);
	}
    
    /*fprintf(stderr, "buffer size      :%i\n",mbuf.size);*/
    
    
    map = (unsigned char*) mmap(0,mbuf.size,PROT_READ|PROT_WRITE,MAP_SHARED,fd,0);
    
    if(map <0)
	{
	    fprintf(stderr,"hvci: initalize, MMAP error %i \n",(int) map);
	    exit(1);
	}
    
    /*
      Setup some sensible defaults for full size pal frame grabbing
    */
    
    gb.format = VIDEO_PALETTE_RGB24;
    gb.frame  = 0;                  
    gb.width = maxWidth;                 
    gb.height = maxHeight;
    
    
    /*
      Get the picture settings (This is experimental on my part)
      
      This will be use by the set_brightness /hue/colour/constrast etc..
      
    */

    if(-1 == ioctl(fd,VIDIOCGPICT,&vpict))
	{
	    fprintf(stderr,"hvci:initalize, ioctl VIDIOCGPICT error\n");
	    exit(1);
	}
    
  


}



/* Select the video source */

void video_set_source(int source)
{
    struct video_channel channel;
    channel.channel = source;
  
    
    if (-1 == ioctl(fd,VIDIOCGCHAN,&channel))
	{
	    fprintf(stderr,"hvci:video_set_source, ioctl VIDIOCGCHAN errror!\n");
	    exit(1);
	}
  
    
}




/*
  Set the dimentions of the image (will only set to the closest possible size)
*/

void video_set_size(int width,int height)
{
 
    fprintf(stderr,"hvci: video_set_size, Please use set zoom instead\n");
    /*
      gb is used in the ioctl which captures the frame
    */              
    gb.width = width;                 
    gb.height = height;
    
}



/*
  The new method of changeing the size of an image is to use set scale.
  This is more portable.  To get the image dimentions, use get width/height.
*/

void video_set_scale(float denom)
{
    
    
    
    /* 
       The scale factor is a fuction of the maximum video size for your area.
       eg for 1/2 size PAL the denom would be set to 2, this would give
       768/2 x 576/2.
    */
    
    if (denom == 0)
	{
	    fprintf(stderr,"hvci: video_set_scale, infinitely  large images not supported.\n");
	    exit(1);
	}
    
    
    currentWidth = (int) rint ( (double) maxWidth/ (double) denom);
    currentHeight = maxHeight/denom;
    
    


    /* 
       now round width
    */

    currentWidth = currentWidth/8;
    currentWidth = currentWidth*8;


    /*
      recalculate Height so as to fix aspect ratio
    */

 

    currentHeight= (maxHeight * currentWidth)/maxWidth;


    /*
      gb is used in the ioctl which captures the frame
    */
    
    gb.width = currentWidth;                 
    gb.height = currentHeight;
    
    /*
      Not conviced how to reliably get weather the hardware supports this 
      size yet so i wont test for it. (dont tell any one, will you...)
      
      Need to use video_capability queiry to get max and min sizes.
      
      
    */
    
    
    
    

}

int video_get_width()
{
    /*
      just return the width currently in use for now...
    */
 
    return currentWidth;
}

int video_get_height()
{
    /*
      just return the height currently in use for now...
    */
    
    return currentHeight;
}



/*
  tidys up open files and maped memory etc.
*/


void video_tidyup()
{
    
    if(map!=NULL) munmap((char*)map,(unsigned int)mbuf.size);
    
    if(fd!=(int)NULL) close(fd);
    
}


/* 
   The following grab functions Initalize the grabbing of a frame and wait for it
   to complete before returning a pointer to the start of the image.
   
   The image is captured at the time this function is called cf. db_grab()
*/


/*
  Grabs a  image 
*/

unsigned char *video_grab()
{
    /*
      Start the capture running
    */
    
    gb.frame=0;
    
    if (-1 == ioctl(fd,VIDIOCMCAPTURE,&gb)) 
	{
	    fprintf(stderr,"hvci: video_grab, ioctl VIDIOCMCAPTURE fault\n");
	    /*  exit(1); */
	}  
    

    /*
      Wait for that frame capture to complete
    */
    
    if (-1 == ioctl(fd,VIDIOCSYNC,&gb.frame)) 
	{
	    
	    fprintf(stderr,"hvci: video_grab, ioctl VIDIOCSYNC fault\n");
	    /* exit(1);*/
	} 
    
    
    return map;
}


/*
  Double buffered grabbing without the confusion.
  Should work just like grab above, but will return a pointer to a frame which
  was caputured earler, whilst starting off the grabbing for the next frame. 
*/

unsigned char *video_db_grab()
{
    
    /*
      Start the capture running
    */
    
    if (-1 == ioctl(fd,VIDIOCMCAPTURE,&gb)) 
	{
	    fprintf(stderr,"hvci: video_db_grab, ioctl VIDIOCMCAPTURE fault\n");
	    exit(1);
	}
    
    /*
      Select the other frame
    */
    
    gb.frame = (!gb.frame)&1;
    
    
    /*
      Sync on the other frame (which should already be captured).
      This will also mark it as free so the V4L driver can reuse the space
    */

    
    if (-1 == ioctl(fd,VIDIOCSYNC,&gb.frame)) 
	{
	    
	    fprintf(stderr,"hvci: video_db_grab, ioctl VIDIOCSYNC fault\n");
	    /*
	      exit(1);
	      
	      Will not exit with a VIDIOSYNC fault as yet, because the 1st one will
	      fail (the caputure was never started.)
	      
	    */
	} 
    
    /*
      Return a pointer the the frame just sync'ed on
    */
    
    return map + mbuf.offsets[gb.frame];
}




void video_set_format(unsigned int format)
{
    
    /*
      set the format variable for the next grab
    */
    
    /*
      Add bounds testing here, not done decause you can use 'undocumented' 
      video modes here if you dare!

      The #defines for the video modes use the same numbering as the v4i ones.
      
    */
      
    gb.format = format;
     
}
