/*-----------------------------------------------------------------------
 * Copyright 1999 Michael Lincoln <mclinc@essex.ac.uk>
 * 
 * This software is provided free of charge.  Permission is hereby given
 * to use, copy, modify and distribute free of charge the software for
 * any purpose provided that this notice is included verbatim in every
 * copy.  The software is provided on an "as is" basis and the authors
 * and the University of Essex have no obligation to provide maintenance,
 * support, updates, enhancements, or modifications.
 * 
 * All express or implied warranties whether statutory or otherwise as to
 * the merchantability or fitness for a particular purpose of the
 * software are, to the maximum extent permitted by law, excluded.
 *
 * In no circumstances whatsoever shall the author or the University of
 * Essex be liable in contract, tort, or otherwise including liability
 * for negligence for any loss whether direct, indirect or otherwise of
 * business, revenue, profits, anticipated savings, wasted expenditure,
 * corruption or destruction of data or for any indirect or consequential
 * loss whatsoever.
 * 
 *
 *------------------------------------------------------------------------
 */




/*
  This provides the interface between the java 'hvci' class and the hvci api previously defined
*/

#include <jni.h> 
#include "hvci_JNI.h"
#include "hvci.h"
#include "stdio.h"


JNIEXPORT void JNICALL Java_hvci_setSize(JNIEnv *env, jobject obj, jfloat scale)

{

    fprintf(stderr,"hvci (JNI): Size (scale) set with %f\n",scale);
    
    video_set_scale(scale);
}


JNIEXPORT void JNICALL Java_hvci_setSource (JNIEnv *env, jobject obj, jint source)

{

    fprintf(stderr,"hvci (JNI): Source set to %i\n",source);
    video_set_source(source);
}


//I think I'm going to repack it into int as java uses this format.
//maybe I'll suport both?

JNIEXPORT jbyteArray JNICALL Java_hvci_grab (JNIEnv *env, jobject obj)
{
    
    unsigned char *frame;
    
    jsize size;
    jbyte *frameElements;
    jbyteArray jframe;
    jboolean isCopy;
        
    int n;

    fprintf(stderr,"hvci (JNI): grab called\n");

    // ask hvci what size frames we're using and what colour palette
    size = video_get_width() * video_get_height();
    
    //now multiply up number of bytes per pixcel (we dont have a get_format 
    //call in hvci yet)

    //assuming rgb24 mode....
    size*=3;

   

    //make new java byte array

    jframe = (*env)->NewByteArray(env,size);

    //this should throw an exception.
    if(!jframe) 
	fprintf(stderr,"hvci (JNI): Unable to allocate java array\n");
    
    //capture a video  frame
    frame = video_grab();

 
    //'pin down' the array and get a refrence to it.
    frameElements = (*env)->GetByteArrayElements(env,jframe,0);

    //copy the frame data (this dose not repack.)
    for(n =0;n<size;n++)
	{
	    *(frameElements+n) = (jbyte) *(frame++); 
	}

    //release it back to the JVM
    (*env)->ReleaseByteArrayElements(env,jframe,frameElements,0);
    
    
   return jframe;

    
}





/*
 * Class:     hvci
 * Method:    getWidth
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_hvci_getWidth(JNIEnv *env, jobject obj)
{
    return video_get_width();
}

/*
 * Class:     hvci
 * Method:    getHeight
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_hvci_getHeight(JNIEnv *env, jobject obj)
{
    return video_get_height();
}



/*
initalize capture drivers
*/

JNIEXPORT void JNICALL Java_hvci_initalize  (JNIEnv *env, jobject obj )
{

    video_initalize(VIDEO_PAL); //change this to also suport NTSC

    //currently we only support 24 bit rgb under java...
    video_set_format(VIDEO_24_RGB);

}


/*
Tidy up before exit
*/

JNIEXPORT void JNICALL Java_hvci_tidyup (JNIEnv *env , jobject obj)
{
    video_tidyup();   
}

