/*-----------------------------------------------------------------------
 * Copyright 1999 Michael Lincoln <mclinc@essex.ac.uk>
 * 
 * This software is provided free of charge.  Permission is hereby given
 * to use, copy, modify and distribute free of charge the software for
 * any purpose provided that this notice is included verbatim in every
 * copy.  The software is provided on an "as is" basis and the authors
 * and the University of Essex have no obligation to provide maintenance,
 * support, updates, enhancements, or modifications.
 * 
 * All express or implied warranties whether statutory or otherwise as to
 * the merchantability or fitness for a particular purpose of the
 * software are, to the maximum extent permitted by law, excluded.
 *
 * In no circumstances whatsoever shall the author or the University of
 * Essex be liable in contract, tort, or otherwise including liability
 * for negligence for any loss whether direct, indirect or otherwise of
 * business, revenue, profits, anticipated savings, wasted expenditure,
 * corruption or destruction of data or for any indirect or consequential
 * loss whatsoever.
 * 
 *
 *------------------------------------------------------------------------
 */





/*   Test harness for hvci                             */
/* Does not  test every possible combination of calls. */
/* Mike Lincoln 1998 */



#include <stdio.h>
#include <unistd.h>


#include "hvci.h"

int writepnm(char *filename,unsigned char * pic,unsigned int width,unsigned int height,int typenum)
{

    FILE *fd;
    size_t size;
    unsigned int i;
    int pnmtype = 0;
    unsigned char  r,g,b;
    short int *short_pic = (short int *) pic;
    short int pixcel;
    
    
    size = width*height;
    
    fprintf(stderr,"Saving %s\n",filename);

    /* open the file*/

    fd = fopen(filename,"w");
    
    if(!fd)
	{
	    fprintf(stderr,"Can not open output file, %s\n",filename);
	    exit(1);
	}
    
    /* use the correct majic number for the pnm */
    
    if(typenum == VIDEO_8_GRAY) pnmtype = 5;
    else if(typenum == VIDEO_24_RGB) pnmtype = 6;
    else if (typenum == VIDEO_32_RGB) pnmtype = 6;
    else if(typenum == VIDEO_16_RGB) pnmtype = 6;
    else 
	{
	    fprintf(stderr,"hvci_test:writepnm, unknown image format, cannot write\n");
	    exit(1);
	}
    

    /* Write the header*/
    
    fprintf(fd,"P%i\n#Written by test harness\n%i %i\n255\n",pnmtype,width,height);
    
    /*
      Write the image, depending on the byte format
    */
  
    if(typenum == VIDEO_8_GRAY)
	fwrite(pic,size,1,fd);/*write the gray scale picture*/
    

 
    if(typenum == VIDEO_16_RGB)
	{
	    for(i=0;i <size;i++)
		{
		  /* unpack R G B (565) and rescale*/
		  
		  pixcel = *short_pic;


		  b = (pixcel &      0x0001f)*8;
		  g = ((pixcel>>5)&  0x0003f)*4;
		  r = ((pixcel>>11)& 0x0001f)*8;
		  
		  short_pic++;

		  /* Write them*/
		    fputc(r,fd);
		    fputc(g,fd);
		    fputc(b,fd);
		 
		}
	}


    if(typenum == VIDEO_24_RGB)
	{
	    
	    for(i=0;i <size;i++)
		{
		    /* Swap BGR to RGB */
		    fputc(*(pic+2),fd);
		    fputc(*(pic+1),fd);
		    fputc(*(pic),fd);
		    pic+=3;
		}
	}
    
    if (typenum == VIDEO_32_RGB)
	{
	    for(i=0;i<size;i++)
		{
		    /*
		      Swap BGR to RGB and miss every 4th byte (format - XBRGXBGR..)
		    */
		    fputc(*(pic+3),fd);
		    fputc(*(pic+2),fd);
		    fputc(*(pic+1),fd);
		    pic+=4;
		}
	}
    
    fclose(fd);
    
    return 1;
}



int main(void)
     
{
    int xsize,ysize;
    char afilename[20];
    int run,frame;
    
    float n;
    unsigned char *mypic;
    
    fprintf(stderr,"About to call initalize\n");
    
    video_initalize(VIDEO_PAL);
    
    fprintf(stderr,"About to set source to composite in\n");
    
    video_set_source(1);
    
    /****************************************************************/
    /*  Format tests                                                */
    /***************************************************************/
    
    
    fprintf(stderr,"About to set size to full PAL\n");
     
    
    video_set_scale(1);
    
    xsize = video_get_width();
    ysize = video_get_height();
    
    
    fprintf(stderr,"About to start format test run...\n");
    
    for(run=0;run<4;run++)
	{
	    /*
	      8 bit gray test
	    */
	    
	    fprintf(stderr,"About to set format to 8 bit gray,see image_gray.pnm\n");
	    
	    video_set_format(VIDEO_8_GRAY);
	    
	    fprintf(stderr,"Grabbing..\n");
	    
	    mypic = video_grab();
	    
	    sprintf(afilename,"image_gray_%i.pnm",run);
	    
	    writepnm(afilename,mypic,xsize,ysize,VIDEO_8_GRAY); /*write an 8 bit gray pnm*/
	    
	    /*
	      16 bit RGB colour RGB test
	    */
	    
	    
	    
	    fprintf(stderr,"About to set to 16 bit RGB (5,6,5)  colour format,see image_RGB16.pnm\n");
	    
	    video_set_format(VIDEO_16_RGB);  /* 16 bit RGB */
	       
	    fprintf(stderr,"Grabbing..\n");
	    
	    mypic = video_grab();
	    
	    
	    sprintf(afilename,"image_RGB16_%i.pnm",run);
	    
	    writepnm(afilename,mypic,xsize,ysize,VIDEO_16_RGB); /*write an 16 colour  pnm*/
	    



	    /*
	      24 bit RGB colour RGB test
	    */
	    
	    
	    
	    fprintf(stderr,"About to set to 24 bit RGB  colour format,see image_RGB24.pnm\n");
	    
	    video_set_format(VIDEO_24_RGB);  /* 24 bit RGB */
	       
	    fprintf(stderr,"Grabbing..\n");
	    
	    mypic = video_grab();
	    
	    
	    sprintf(afilename,"image_RGB24_%i.pnm",run);
	    
	    writepnm(afilename,mypic,xsize,ysize,VIDEO_24_RGB); /*write an 24 colour  pnm*/
	    
	    
	    /*
	      32 bit XRGB colour test
	    */
	    
	    
	    fprintf(stderr,"About to set to 32 bit XRGB  colour format,see image_RGB32.pnm\n");
	    

	    video_set_format(VIDEO_32_RGB);  /* 24 bit RGB */
	    
	    fprintf(stderr,"Grabbing..\n");
	     
	    mypic = video_grab();
	    
	    
	    sprintf(afilename,"image_RGB32_%i.pnm",run);
	    
	    writepnm(afilename,mypic,xsize,ysize,VIDEO_32_RGB); /*write an 24 colour  pnm*/
	    
	    

	}

  
  
    /**************************************************************************/
    /*Size Tests*/
      
    /*
      Reset format
    */
    
    fprintf(stderr,"About to set to 24 bit RGB  colour format,see image_RGB24.pnm\n");
    
    video_set_format(VIDEO_24_RGB);  /* 24 bit RGB */
    
    fprintf(stderr,"About to test grabing diffent frame sizes\n");
    fprintf(stderr,"  See image_sizeX.pnm where x is palsize/1 to palsize/13\n");
    
    for(n = 1.0 ; n<10.0 ; n = n + 0.1)
	{
	    
	    fprintf(stderr,"About to set scale  to %f\n",(double)n);
	    video_set_scale(n);
	    
	    
	    xsize = video_get_width();
	    ysize = video_get_height();
	    
	    fprintf(stderr,"Returned size was %i x %i\n",xsize,ysize);
	    
	    fprintf(stderr,"About to grab frame\n");
	    
	    mypic = video_grab();
	    
	    sprintf(afilename,"image_size%2.2f.pnm",(double)n);
	    
	    writepnm(afilename,mypic,xsize,ysize,VIDEO_24_RGB); /*write a colour pnm*/
	  
	  
	  
    }
  
  
  
  /********************************************************************/
  /* Double buffered grabbing tests*/
  
  fprintf(stderr,"About to set size to  half PAL \n");
  
  video_set_scale(2);
  
  
  xsize = video_get_width();
  ysize = video_get_height();
  
  fprintf(stderr,"About to set format to 8 bit gray\n");
  
  video_set_format(1);
  
  fprintf(stderr,"About to capture 3 secounds of video into image_filmX.pnm where X is 0 to 75\n");
  
  for(frame =0;frame<75;frame++)
      {
      
	  mypic = video_db_grab();
	  
	  sprintf(afilename,"image_film%i.pnm",frame);
	  
	  writepnm(afilename,mypic,xsize,ysize,VIDEO_8_GRAY); /*write a 8bit gray pnm*/
      }
  
  
  
  
  
  /***************************************************************/
  
  
  
  fprintf(stderr,"Finished, about to call tidy up\n");
  
  video_tidyup();
  
  return 1; 
  
}
